@@ -18,25 +18,142 @@
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
+#include <getopt.h>
 
 #include "commands.h"
 #include "utils.h"
+#include "help.h"
 
 #define USAGE_SHORT		1U
 #define USAGE_LONG		2U
 #define USAGE_OPTIONS		4U
 #define USAGE_LISTING		8U
 
+static char argv0_buf[ARGV0_BUF_SIZE] = "btrfs";
+
+const char *get_argv0_buf(void)
+{
+	return argv0_buf;
+}
+
+void fixup_argv0(char **argv, const char *token)
+{
+	int len = strlen(argv0_buf);
+
+	snprintf(argv0_buf + len, sizeof(argv0_buf) - len, " %s", token);
+	argv[0] = argv0_buf;
+}
+
+void set_argv0(char **argv)
+{
+	strncpy(argv0_buf, argv[0], sizeof(argv0_buf));
+	argv0_buf[sizeof(argv0_buf) - 1] = 0;
+}
+
+int check_argc_exact(int nargs, int expected)
+{
+	if (nargs < expected)
+		fprintf(stderr, "%s: too few arguments\n", argv0_buf);
+	if (nargs > expected)
+		fprintf(stderr, "%s: too many arguments\n", argv0_buf);
+
+	return nargs != expected;
+}
+
+int check_argc_min(int nargs, int expected)
+{
+	if (nargs < expected) {
+		fprintf(stderr, "%s: too few arguments\n", argv0_buf);
+		return 1;
+	}
+
+	return 0;
+}
+
+int check_argc_max(int nargs, int expected)
+{
+	if (nargs > expected) {
+		fprintf(stderr, "%s: too many arguments\n", argv0_buf);
+		return 1;
+	}
+
+	return 0;
+}
+
+/*
+ * Preprocess @argv with getopt_long to reorder options and consume the "--"
+ * option separator.
+ * Unknown short and long options are reported, optionally the @usage is printed
+ * before exit.
+ */
+void clean_args_no_options(int argc, char *argv[], const char * const *usagestr)
+{
+	static const struct option long_options[] = {
+		{NULL, 0, NULL, 0}
+	};
+
+	while (1) {
+		int c = getopt_long(argc, argv, "", long_options, NULL);
+
+		if (c < 0)
+			break;
+
+		switch (c) {
+		default:
+			if (usagestr)
+				usage(usagestr);
+		}
+	}
+}
+
+/*
+ * Same as clean_args_no_options but pass through arguments that could look
+ * like short options. Eg. reisze which takes a negative resize argument like
+ * '-123M' .
+ *
+ * This accepts only two forms:
+ * - "-- option1 option2 ..."
+ * - "option1 option2 ..."
+ */
+void clean_args_no_options_relaxed(int argc, char *argv[], const char * const *usagestr)
+{
+	if (argc <= 1)
+		return;
+
+	if (strcmp(argv[1], "--") == 0)
+		optind = 2;
+}
+
 static int do_usage_one_command(const char * const *usagestr,
 				unsigned int flags, FILE *outf)
 {
 	int pad = 4;
+	const char *prefix = "usage: ";
+	const char *pad_listing = "    ";
 
 	if (!usagestr || !*usagestr)
 		return -1;
 
-	fprintf(outf, "%s%s", (flags & USAGE_LISTING) ? "    " : "usage: ",
-		*usagestr++);
+	if (flags & USAGE_LISTING)
+		prefix = pad_listing;
+
+	fputs(prefix, outf);
+	if (strchr(*usagestr, '\n') == NULL) {
+		fputs(*usagestr, outf);
+	} else {
+		const char *c = *usagestr;
+		const char *nprefix = "       ";
+
+		if (flags & USAGE_LISTING)
+			nprefix = pad_listing;
+
+		for (c = *usagestr; *c; c++) {
+			fputc(*c, outf);
+			if (*c == '\n')
+				fputs(nprefix, outf);
+		}
+	}
+	usagestr++;
 
 	/* a short one-line description (mandatory) */
 	if ((flags & USAGE_SHORT) == 0)
@@ -121,6 +238,7 @@ void usage_command(const struct cmd_struct *cmd, int full, int err)
 	usage_command_usagestr(cmd->usagestr, cmd->token, full, err);
 }
 
+__attribute__((noreturn))
 void usage(const char * const *usagestr)
 {
 	usage_command_usagestr(usagestr, NULL, 1, 1);
@@ -148,6 +266,8 @@ static void usage_command_group_internal(const struct cmd_group *grp, int full,
 
 			usage_command_internal(cmd->usagestr, cmd->token, full,
 					       1, cmd->flags & CMD_ALIAS, outf);
+			if (cmd->flags & CMD_ALIAS)
+				putchar('\n');
 			continue;
 		}
 
@@ -226,6 +346,7 @@ void usage_command_group(const struct cmd_group *grp, int full, int err)
 		fprintf(outf, "%s\n", grp->infostr);
 }
 
+__attribute__((noreturn))
 void help_unknown_token(const char *arg, const struct cmd_group *grp)
 {
 	fprintf(stderr, "%s: unknown token '%s'\n", get_argv0_buf(), arg);
@@ -233,6 +354,7 @@ void help_unknown_token(const char *arg, const struct cmd_group *grp)
 	exit(1);
 }
 
+__attribute__((noreturn))
 void help_ambiguous_token(const char *arg, const struct cmd_group *grp)
 {
 	const struct cmd_struct *cmd = grp->commands;
@@ -260,12 +382,3 @@ void help_command_group(const struct cmd_group *grp, int argc, char **argv)
 	usage_command_group(grp, full, 0);
 }
 
-int prefixcmp(const char *str, const char *prefix)
-{
-	for (; ; str++, prefix++)
-		if (!*prefix)
-			return 0;
-		else if (*str != *prefix)
-			return (unsigned char)*prefix - (unsigned char)*str;
-}
-
