@@ -88,24 +88,6 @@ void free_recover_superblock(struct btrfs_recover_superblock *recover)
 	}
 }
 
-static int check_super(u64 bytenr, struct btrfs_super_block *sb)
-{
-	int csum_size = btrfs_super_csum_size(sb);
-	char result[csum_size];
-	u32 crc = ~(u32)0;
-
-	if (btrfs_super_bytenr(sb) != bytenr)
-		return 0;
-	if (sb->magic != cpu_to_le64(BTRFS_MAGIC))
-		return 0;
-
-	crc = btrfs_csum_data(NULL, (char *)sb + BTRFS_CSUM_SIZE,
-			crc, BTRFS_SUPER_INFO_SIZE - BTRFS_CSUM_SIZE);
-	btrfs_csum_final(crc, result);
-
-	return !memcmp(sb, &result, csum_size);
-}
-
 static int add_superblock_record(struct btrfs_super_block *sb, char *fname,
 			u64 bytenr, struct list_head *head)
 {
@@ -133,24 +115,20 @@ read_dev_supers(char *filename, struct btrfs_recover_superblock *recover)
 	int i, ret, fd;
 	u8 buf[BTRFS_SUPER_INFO_SIZE];
 	u64 max_gen, bytenr;
+	struct btrfs_super_block *sb = (struct btrfs_super_block *)buf;
+
 	/* just ignore errno that were set in btrfs_scan_fs_devices() */
 	errno = 0;
 
-	struct btrfs_super_block *sb = (struct btrfs_super_block *)buf;
-
 	fd = open(filename, O_RDONLY);
 	if (fd < 0)
 		return -errno;
 
 	for (i = 0; i < BTRFS_SUPER_MIRROR_MAX; i++) {
 		bytenr = btrfs_sb_offset(i);
-		ret = pread64(fd, buf, sizeof(buf), bytenr);
-		if (ret < sizeof(buf)) {
-			ret = -errno;
-			goto out;
-		}
-		ret = check_super(bytenr, sb);
-		if (ret) {
+
+		ret = btrfs_read_dev_super(fd, sb, bytenr, SBREAD_DEFAULT);
+		if (!ret) {
 			ret = add_superblock_record(sb, filename, bytenr,
 							&recover->good_supers);
 			if (ret)
@@ -158,13 +136,18 @@ read_dev_supers(char *filename, struct btrfs_recover_superblock *recover)
 			max_gen = btrfs_super_generation(sb);
 			if (max_gen > recover->max_generation)
 				recover->max_generation = max_gen;
-		} else {
+		} else if (ret != -ENOENT){
+			/*
+			 * Skip superblock which doesn't exist, only adds
+			 * really corrupted superblock
+			 */
 			ret = add_superblock_record(sb, filename, bytenr,
 						&recover->bad_supers);
 			if (ret)
 				goto out;
 		}
 	}
+	ret = 0;
 out:
 	close(fd);
 	return ret;
@@ -194,28 +177,6 @@ static int read_fs_supers(struct btrfs_recover_superblock *recover)
 	return 0;
 }
 
-static struct super_block_record *recover_get_good_super(
-				struct btrfs_recover_superblock *recover)
-{
-	struct super_block_record *record;
-	record = list_entry(recover->good_supers.next,
-				struct super_block_record, list);
-	return record;
-}
-
-static void print_all_devices(struct list_head *devices)
-{
-	struct btrfs_device *dev;
-
-	printf("All Devices:\n");
-	list_for_each_entry(dev, devices, dev_list) {
-		printf("\t");
-		printf("Device: id = %llu, name = %s\n",
-			dev->devid, dev->name);
-	}
-	printf("\n");
-}
-
 static void print_super_info(struct super_block_record *record)
 {
 	printf("\t\tdevice name = %s\n", record->device_name);
@@ -310,7 +271,9 @@ int btrfs_recover_superblocks(const char *dname,
 			goto no_recover;
 		}
 	}
-	record = recover_get_good_super(&recover);
+	record = list_first_entry(&recover.good_supers,
+				  struct super_block_record, list);
+
 	root = open_ctree(record->device_name, record->bytenr,
 			  OPEN_CTREE_RECOVER_SUPER | OPEN_CTREE_WRITES);
 	if (!root) {
@@ -319,7 +282,7 @@ int btrfs_recover_superblocks(const char *dname,
 	}
 	/* reset super_bytenr in order that we will rewrite all supers */
 	root->fs_info->super_bytenr = BTRFS_SUPER_INFO_OFFSET;
-	ret = write_all_supers(root);
+	ret = write_all_supers(root->fs_info);
 	if (!ret)
 		ret = 2;
 	else
