@@ -10,7 +10,7 @@ check_default_id()
 	fi
 }
 
-source "$TOP/tests/common"
+source "$TEST_TOP/common"
 
 check_prereq mkfs.btrfs
 check_prereq btrfs
