@@ -2,7 +2,7 @@
 
 # iterate over all fuzzed images and run 'btrfs check'
 
-source $TOP/tests/common
+source "$TEST_TOP/common"
 
 setup_root_helper
 check_prereq btrfs
@@ -15,6 +15,6 @@ check_image() {
 	run_mayfail $TOP/btrfs check "$image"
 }
 
-check_all_images $TOP/tests/fuzz-tests/images
+check_all_images $TEST_TOP/fuzz-tests/images
 
 exit 0
