@@ -4,18 +4,40 @@
 
 LANG=C
 SCRIPT_DIR=$(dirname $(readlink -f "$0"))
-TOP=$(readlink -f "$SCRIPT_DIR/../")
+if [ -z "$TOP" ]; then
+	TOP=$(readlink -f "$SCRIPT_DIR/../")
+	if [ -f "$TOP/configure.ac" ]; then
+		# inside git
+		TEST_TOP="$TOP/tests/"
+		INTERNAL_BIN="$TOP"
+	else
+		# external, defaults to system binaries
+		TOP=$(dirname `which btrfs`)
+		TEST_TOP="$SCRIPT_DIR"
+		INTERNAL_BIN="$TEST_TOP"
+	fi
+else
+	# assume external, TOP set from commandline
+	TEST_TOP="$SCRIPT_DIR"
+	INTERNAL_BIN="$TEST_TOP"
+fi
+if ! [ -x "$TOP/btrfs" ]; then
+	echo "ERROR: cannot execute btrfs from TOP=$TOP"
+	exit 1
+fi
 TEST_DEV=${TEST_DEV:-}
-RESULTS="$TOP/tests/misc-tests-results.txt"
-IMAGE="$TOP/tests/test.img"
+RESULTS="$TEST_TOP/misc-tests-results.txt"
+IMAGE="$TEST_TOP/test.img"
 
-source "$TOP/tests/common"
+source "$TEST_TOP/common"
 
+export INTERNAL_BIN
+export TEST_TOP
 export TOP
 export RESULTS
 export LANG
-export TEST_DEV
 export IMAGE
+export TEST_DEV
 
 rm -f "$RESULTS"
 
@@ -31,7 +53,7 @@ check_kernel_support
 
 # The tests are driven by their custom script called 'test.sh'
 
-for i in $(find "$TOP/tests/misc-tests" -maxdepth 1 -mindepth 1 -type d	\
+for i in $(find "$TEST_TOP/misc-tests" -maxdepth 1 -mindepth 1 -type d	\
 	${TEST:+-name "$TEST"} | sort)
 do
 	echo "    [TEST/misc]   $(basename $i)"
@@ -46,5 +68,5 @@ do
 			_fail "test failed for case $(basename $i)"
 		fi
 	fi
-	cd "$TOP"
+	cd "$TEST_TOP"
 done
