@@ -1,19 +1,19 @@
 #!/bin/bash
 # test btrfstune uuid rewriting options
 
-source $TOP/tests/common
+source "$TEST_TOP/common"
 
-check_prereq btrfs-debug-tree
-check_prereq btrfs-show-super
 check_prereq mkfs.btrfs
 check_prereq btrfstune
+check_prereq btrfs
+
 prepare_test_dev
 
 get_fs_uuid() {
 	local image
 
 	image="$1"
-	run_check_stdout $TOP/btrfs-show-super "$image" | \
+	run_check_stdout $TOP/btrfs inspect-internal dump-super "$image" | \
 		grep '^fsid' | awk '{print $2}'
 }
 
@@ -25,15 +25,15 @@ test_uuid_random()
 
 	run_check $SUDO_HELPER $TOP/mkfs.btrfs -f \
 		--uuid $origuuid \
-		--rootdir $TOP/Documentation \
+		--rootdir $INTERNAL_BIN/Documentation \
 		$TEST_DEV
-	run_check $TOP/btrfs-show-super "$TEST_DEV"
+	run_check $TOP/btrfs inspect-internal dump-super "$TEST_DEV"
 	currentfsid=$(run_check_stdout $TOP/btrfstune -f -u $TEST_DEV | \
 		grep -i 'current fsid:' | awk '{print $3}')
 	if ! [ $currentfsid = $origuuid ]; then
 		_fail "FAIL: current UUID mismatch"
 	fi
-	run_check $TOP/btrfs-show-super "$TEST_DEV"
+	run_check $TOP/btrfs inspect-internal dump-super "$TEST_DEV"
 	run_check $SUDO_HELPER $TOP/btrfs check $TEST_DEV
 }
 
@@ -47,12 +47,12 @@ test_uuid_user()
 
 	run_check $SUDO_HELPER $TOP/mkfs.btrfs -f \
 		--uuid $origuuid \
-		--rootdir $TOP/Documentation \
+		--rootdir $INTERNAL_BIN/Documentation \
 		$TEST_DEV
-	run_check $TOP/btrfs-show-super "$TEST_DEV"
+	run_check $TOP/btrfs inspect-internal dump-super "$TEST_DEV"
 	run_check $TOP/btrfstune -f -U $newuuid \
 		$TEST_DEV
-	# btrfs-show-super is called within get_fs_uuid
+	# btrfs inspect-internal dump-super is called within get_fs_uuid
 	fsid=$(get_fs_uuid $TEST_DEV)
 	if ! [ $fsid = $newuuid ]; then
 		_fail "FAIL: UUID not rewritten"
