@@ -2,7 +2,7 @@
 #
 # test label settings
 
-source $TOP/tests/common
+source "$TEST_TOP/common"
 
 check_prereq mkfs.btrfs
 check_prereq btrfs
