@@ -1,7 +1,12 @@
 #!/bin/bash
 # recognize partitioned loop devices
 
-source $TOP/tests/common
+source "$TEST_TOP/common"
+
+if ! losetup --help | grep -q 'partscan'; then
+	_not_run "losetup --partscan not available"
+	exit 0
+fi
 
 check_prereq mkfs.btrfs
 
@@ -24,3 +29,4 @@ done
 # cleanup
 run_check $SUDO_HELPER losetup -d $loopdev
 run_check truncate -s0 img
+rm img
