@@ -1,7 +1,7 @@
 #!/bin/bash
 # iterate over nodesize and sectorsize combinations
 
-source $TOP/tests/common
+source "$TEST_TOP/common"
 
 check_prereq mkfs.btrfs
 check_prereq btrfs
@@ -12,7 +12,7 @@ prepare_test_dev
 test_mkfs_single()
 {
 	run_check $SUDO_HELPER $TOP/mkfs.btrfs -f "$@" $TEST_DEV
-	run_check $TOP/btrfs inspect-internal dump-super $TEST_DEV
+	run_check $SUDO_HELPER $TOP/btrfs inspect-internal dump-super $TEST_DEV
 	run_check $SUDO_HELPER $TOP/btrfs check $TEST_DEV
 }
 
